# -*- coding: utf-8 -*-
#
# Author: Timur Gilmullin

# This module initialize standard python logging system.


import sys
import logging.handlers


# initialize Main Parent Logger:
UniLogger = logging.getLogger("UniLogger")
formatString = "%(filename)-20sL:%(lineno)-5d%(levelname)-8s[%(asctime)s] %(message)s"
formatter = logging.Formatter(formatString)
sys.stderr = sys.stdout


def SetLevel(vLevel='ERROR'):
    """
    This procedure setting up UniLogger verbosity level.
    """
    UniLogger.level = logging.NOTSET

    if isinstance(vLevel, str):
        if vLevel == '5' or vLevel.upper() == 'CRITICAL':
            UniLogger.level = logging.CRITICAL

        elif vLevel == '4' or vLevel.upper() == 'ERROR':
            UniLogger.level = logging.ERROR

        elif vLevel == '3' or vLevel.upper() == 'WARNING':
            UniLogger.level = logging.WARNING

        elif vLevel == '2' or vLevel.upper() == 'INFO':
            UniLogger.level = logging.INFO

        elif vLevel == '1' or vLevel.upper() == 'DEBUG':
            UniLogger.level = logging.DEBUG


class LevelFilter(logging.Filter):
    """
    Class using to set up log level filtering.
    """

    def __init__(self, level):
        super().__init__()
        self.level = level

    def filter(self, record):
        return record.levelno >= self.level


def EnableLogger(logFile, parentHandler=UniLogger, useFormat=formatter):
    """
    Adding new file logger with rotation.
    """
    # logHandler = logging.FileHandler(logFile)
    maxSizeBytes = 50 * 1024 * 1024  # 5Mb log rotate by default
    logHandler = logging.handlers.RotatingFileHandler(logFile, encoding="UTF-8", maxBytes=maxSizeBytes, backupCount=4)
    logHandler.level = logging.DEBUG  # set up DEBUG verbosity level by default for file logging
    logHandler.addFilter(LevelFilter(logging.DEBUG))

    if useFormat:
        logHandler.setFormatter(useFormat)

    else:
        logHandler.setFormatter(formatter)

    parentHandler.addHandler(logHandler)

    return logHandler


def DisableLogger(handler, parentHandler=UniLogger):
    """
    Disable given file logger.
    """
    if handler:
        handler.flush()
        handler.close()

    if handler in parentHandler.handlers:
        parentHandler.removeHandler(handler)


# --- Main init:

SetLevel('DEBUG')  # set up DEBUG verbosity level by default for UniLogger

streamHandler = logging.StreamHandler()  # initialize STDOUT UniLogger
streamHandler.setFormatter(formatter)  # set formatter for STDOUT UniLogger
streamHandler.level = logging.INFO  # set up INFO verbosity level by default for STDOUT UniLogger
UniLogger.addHandler(streamHandler)  # adding STDOUT UniLogger handler to Parent UniLogger

# fileLogHandler = EnableLogger(logFile='log.txt', parentHandler=UniLogger, useFormat=formatter)  # add logging to file

sepWide = '-' * 120  # long-long log separator
sepLong = '-' * 80  # long log separator
sepShort = '-' * 40  # short log separator
sepLine = '=--=' * 20  # log part separator



# -*- coding: utf-8 -*-
# Author: Timur Gilmullin

# Build with Travis CI


from setuptools import setup
import os

__version__ = "1.1"

devStatus = "4 - Beta"

if "TRAVIS_BUILD_NUMBER" in os.environ and "TRAVIS_BRANCH" in os.environ:
    print("This is TRAVIS-CI build")
    print("TRAVIS_BUILD_NUMBER = {}".format(os.environ["TRAVIS_BUILD_NUMBER"]))
    print("TRAVIS_BRANCH = {}".format(os.environ["TRAVIS_BRANCH"]))

    __version__ += ".{}{}".format(
        "" if "release" in os.environ["TRAVIS_BRANCH"] or os.environ["TRAVIS_BRANCH"] == "master" else "dev",
        os.environ["TRAVIS_BUILD_NUMBER"],
    )

    devStatus = "5 - Production/Stable" if "release" in os.environ["TRAVIS_BRANCH"] or os.environ["TRAVIS_BRANCH"] == "master" else devStatus

else:
    print("This is local build")
    __version__ += ".dev0"  # set version as major.minor.localbuild if local build: python setup.py install

print("MT4ForexParser build version = {}".format(__version__))

setup(
    name="mt4forexparser",

    version=__version__,

    description="Read forex data in MetaTrader 4 .hst-format and convert into .csv file and pandas dataframe. Also you can draw an interactive chart.",

    long_description="GitHub Pages: https://tim55667757.github.io/MT4ForexParser",

    license="MIT",

    author="Timur Gilmullin",

    author_email="tim55667757@gmail.com",

    url="https://github.com/Tim55667757/MT4ForexParser/",

    download_url="https://github.com/Tim55667757/MT4ForexParser.git",

    entry_points={"console_scripts": ["mt4forexparser = mt4forexparser.MT4ForexParser:Main"]},

    classifiers=[
        "Development Status :: {}".format(devStatus),
        "Environment :: Console",
        "Intended Audience :: Financial and Insurance Industry",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
    ],

    keywords=[
        "forex",
        "broker",
        "history",
        "csv",
        "stock",
        "trade",
        "candlestick",
        "chart",
        "parser",
        "MetaTrader",
        "MetaTrader4",
    ],

    tests_require=[
        "pytest>=6.2.2",
        "pandas>=1.2.2",
        "pricegenerator>=1.2.36",
    ],

    install_requires=[
        "pandas>=1.2.2",
        "pricegenerator>=1.2.36",
    ],

    packages=[
        "mt4forexparser",
    ],

    zip_safe=True,
)



# -*- coding: utf-8 -*-
# Author: Timur Gilmullin


# Module: MT4 Forex History Parser.
# Read forex data in MetaTrader 4 .hst-format and convert into .csv file and pandas dataframe.


import os
import sys
sys.path.append("..")
from datetime import datetime
import numpy as np
import pandas as pd
from argparse import ArgumentParser

import mt4forexparser.UniLogger as uLog
import traceback as tb

from pricegenerator import PriceGenerator as pg


# --- Common technical parameters:

uLogger = uLog.UniLogger
uLogger.level = 10  # debug level by default
uLogger.handlers[0].level = 20  # info level by default for STDOUT
uLogger.handlers[1].level = 50  # disable duplicate logging added by PriceGenerator
# uLogger.handlers[1].level = 10  # debug level by default for log.txt


def MT4ParseToPD(historyFile, outputFile=None):
    """
    Read and parse MetaTrader 4 .hst file. Save to .csv if needed. Return pandas dataframe.
    :param historyFile: full path to MetaTrader 4 .hst-file.
    :param outputFile: full path to .csv output file. If not defined then only return pandas dataframe.
    """
    if historyFile is None:
        raise Exception("Path to MetaTrader 4 history file must be define!")

    uLogger.debug("MT4 history file: [{}]".format(os.path.abspath(historyFile)))

    with open(historyFile, "rb") as fH:
        version = np.frombuffer(fH.read(148)[:4], "i4")
        uLogger.debug("MT4 history file format version: {}".format(version))

        if version == 400:
            dType = [("datetime", "u4"), ("open", "f8"), ("low", "f8"), ("high", "f8"), ("close", "f8"), ("volume", "f8")]
            df = pd.DataFrame(np.frombuffer(fH.read(), dtype=dType))

        elif version == 401:
            dType = [("datetime", "u8"), ("open", "f8"), ("high", "f8"), ("low", "f8"), ("close", "f8"), ("volume", "i8"), ("s", "i4"), ("r", "i8")]
            data = np.frombuffer(fH.read(), dtype=dType)
            df = pd.DataFrame(data).drop(axis=1, labels=["s", "r"])

        else:
            raise Exception("MetaTrader 4 history file has unsupported version: {}".format(version))

        df["datetime"] = pd.to_datetime(df["datetime"], unit="s")
        df["date"] = df["datetime"].apply(lambda x: x.strftime("%Y.%m.%d"))
        df["time"] = df["datetime"].apply(lambda x: x.strftime("%H:%M"))
        df["volume"] = df["volume"].astype(int)

        df = df.drop(axis=1, labels=["datetime"])[["date", "time", "open", "high", "low", "close", "volume"]]

    uLogger.info("It was read {} rows from file [{}]".format(len(df), os.path.abspath(historyFile)))
    uLogger.info("Showing last 3 rows:")
    for line in pd.DataFrame.to_string(
        df[["date", "time", "open", "high", "low", "close", "volume"]][-3:],
        max_cols=20
    ).split("\n"):
        uLogger.info(line)

    if outputFile is not None:
        df.to_csv(outputFile, index=False, header=False)
        uLogger.info("Forex history saved to .csv-formatted file [{}]".format(os.path.abspath(outputFile)))

    else:
        uLogger.debug("--output key is not defined. Parsed history file not saved to .csv-file.")

    return df


def Render(prices: pd.DataFrame, name="Example instrument", show=True):
    """
    Render interactive chart using PriceGenerator library.
    :param prices: Pandas dataframe with prices.
    :param name: Instrument's name for chart title.
    :param show: Show in browser immediately if True.
    """
    chart = pg.PriceGenerator()
    prices["datetime"] = pd.to_datetime(prices["date"] + " " + prices["time"])
    chart.ticker = os.path.basename(name)
    chart.prices = prices
    chart.RenderBokeh(viewInBrowser=show)


def ParseArgs():
    """
    Function get and parse command line keys.
    """
    parser = ArgumentParser()  # command-line string parser

    parser.description = "Metatrader 4 forex history parser. Read, parse and save history as .csv-file or pandas dataframe. Also you can draw an interactive chart. See examples: https://tim55667757.github.io/MT4ForexParser"
    parser.usage = "mt4forexparser [some options] [one command]"

    # options:
    parser.add_argument("--mt4-history", type=str, required=True, help="Option (required): full path to Metatrader 4 forex history file.")
    parser.add_argument("--output", type=str, default=None, help="Option: full path to .csv output file. Default is None, mean that returns only pandas dataframe.")
    parser.add_argument("--debug-level", type=int, default=20, help="Option: showing STDOUT messages of minimal debug level, e.g. 10 = DEBUG, 20 = INFO, 30 = WARNING, 40 = ERROR, 50 = CRITICAL.")

    # commands:
    parser.add_argument("--parse", action="store_true", help="Command: read, parse and save mt4-history as pandas dataframe or .csv-file if --output is define.")
    parser.add_argument("--render", action="store_true", help="Command: use PriceGenerator module to render interactive chart from parsed data. This key only used with --parse key.")

    cmdArgs = parser.parse_args()
    return cmdArgs


def Main():
    """
    Main function for reading, parsing and saving mt4-history as .csv-file or pandas dataframe.
    """
    history = None
    output = None

    args = ParseArgs()  # get and parse command-line parameters
    exitCode = 0

    if args.debug_level:
        uLogger.level = 10  # always debug level by default
        uLogger.handlers[0].level = args.debug_level  # level for STDOUT
        uLogger.handlers[1].level = 50  # disable duplicate logging added by PriceGenerator
        # uLogger.handlers[1].level = 10  # always debug level for log.txt

    start = datetime.now()
    uLogger.debug("MT4 parser started: {}".format(start.strftime("%Y-%m-%d %H:%M:%S")))

    try:
        # --- set options:

        if args.mt4_history:
            history = args.mt4_history

        if args.output:
            output = args.output

        # --- do one command:

        if args.parse:
            parsedData = MT4ParseToPD(history, output)

            if args.render and parsedData is not None:
                Render(prices=parsedData, name=history, show=True)

        else:
            raise Exception("One of the possible commands must be selected! See: python MT4ForexParser.py --help")

    except Exception:
        exc = tb.format_exc().split("\n")
        for line in exc:
            if line:
                uLogger.debug(line)
        exitCode = 255

    finally:
        finish = datetime.now()

        if exitCode == 0:
            uLogger.debug("All MT4 parser operations are finished success (summary code is 0).")

        else:
            uLogger.error("An errors occurred during the work! See full debug log with --debug-level 10. Summary code: {}".format(exitCode))

        uLogger.debug("MT4 parser work duration: {}".format(finish - start))
        uLogger.debug("MT4 parser work finished: {}".format(finish.strftime("%Y-%m-%d %H:%M:%S")))

        sys.exit(exitCode)


if __name__ == "__main__":
    Main()
